/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.sql.framework.ui.editor.property.impl;

import java.util.ArrayList;
import java.util.List;

import org.netbeans.modules.sql.framework.ui.editor.property.IElement;
import org.netbeans.modules.sql.framework.ui.editor.property.IOptionProperty;
import org.netbeans.modules.sql.framework.ui.editor.property.IPropertyCustomizer;


/**
 * @author Ritesh Adval
 * @version $Revision$
 */
public class OptionPropertySupport extends BasicPropertySupport implements IOptionProperty {

    private int defaultOption = 0;

    private ArrayList<IElement> optionList = new ArrayList<IElement>();

    public OptionPropertySupport() {
        super();
    }

    /** Creates a new instance of OptionPropertySupport */
    public OptionPropertySupport(String valueType) {
        super(valueType);
    }

    /**
     * add a element in the node
     * 
     * @param element element to add
     */
    @Override
    public void add(IElement element) {
        element.setParent(this);
        optionList.add(element);
    }

    public int getDefaultOption() {
        return defaultOption;
    }

    public BasicOption getOptionForDisplayName(String displayName) {
        for (int i = 0; i < optionList.size(); i++) {
            BasicOption option = (BasicOption) optionList.get(i);
            if (displayName != null && displayName.equals(option.getDisplayName())) {
                return option;
            }
        }
        return null;
    }

    public BasicOption getOptionForValue(String value) {
        for (int i = 0; i < optionList.size(); i++) {
            BasicOption option = (BasicOption) optionList.get(i);
            if (value != null && value.equals(option.getValue())) {
                return option;
            }
        }
        return null;
    }

    public List getOptions() {
        return optionList;
    }

    public void setDefaultOption(String defOption) {
        defaultOption = Integer.parseInt(defOption);
    }

    /**
     * set the optional property editor which can be used to edit this property
     * 
     * @return property editor
     */
    @Override
    public void setEditorClass(String editorClass) {
        super.setEditorClass(editorClass);
        if (this.getPropertyEditor() instanceof OptionListEditor) {
            ((OptionListEditor) this.getPropertyEditor()).setProperty(this);
        }
    }

    /**
     * set the property customizer
     * 
     * @param customizer customizer
     */
    @Override
    public void setPropertyCustomizer(IPropertyCustomizer customizer) {
        super.setPropertyCustomizer(customizer);
        if (customizer == null) {
            return;
        }

        if (this.getPropertyEditor() instanceof OptionListEditor) {
            OptionListEditor listEditor = (OptionListEditor) this.getPropertyEditor();
            listEditor.setCustomOptions(customizer.getOptions());

        }
    }

    // override set value to handle Integer options; if optionProperty type is integer we
    // want to convert passed String optiong value to integer
    @Override
    public void setValue(Object obj) throws java.lang.IllegalAccessException, java.lang.IllegalArgumentException,
            java.lang.reflect.InvocationTargetException {
        if (this.getValueType() == Integer.class && obj instanceof String) {
            try {
                super.setValue(new Integer((String) obj));
            } catch (NumberFormatException ex) {
                ex.printStackTrace();
                // log me
            }
        } else {
            super.setValue(obj);
        }
    }
}

