/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 1997-2009 Sun Microsystems, Inc.
 */

package org.netbeans.modules.wlm.bridge.wizard;

import java.util.List;
import org.netbeans.modules.wlm.bridge.model.BPELVariablesTreeModel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import org.netbeans.modules.soa.dndbridge.api.VariableType;
import org.netbeans.modules.xml.wsdl.model.Message;

public final class WLMVisualPanel2 extends JPanel {
    static String INPUT_MESSAGE_TYPE = "input_message_type";
    static String OUTPUT_MESSAGE_TYPE = "output_message_type";
    static String FAULT_MESSAGE_TYPE = "fault_message_type";

    private BPELVariablesTreeModel bpelVariablesInputTreeModel;
    private BPELVariablesTreeModel bpelVariablesOutputTreeModel;
    private BPELVariablesTreeModel bpelVariablesFaultTreeModel;
    DefaultMutableTreeNode inputTreeTopTreeNode;
    DefaultMutableTreeNode outputTreeTopTreeNode;
    DefaultMutableTreeNode faultTreeTopTreeNode;
    Message inputMessageType;
    Message outputMessageType;
    Message faultMessageType;

    /** Creates new form WLMVisualPanel2 */
    public WLMVisualPanel2() {
        initComponents();
    }

    WLMVisualPanel2(List<VariableType> messages) {
        bpelVariablesInputTreeModel = new BPELVariablesTreeModel(messages);
        bpelVariablesOutputTreeModel = new BPELVariablesTreeModel(messages);
        bpelVariablesFaultTreeModel = new BPELVariablesTreeModel(messages);
        inputTreeTopTreeNode = (DefaultMutableTreeNode) bpelVariablesInputTreeModel.getRoot();
        outputTreeTopTreeNode = (DefaultMutableTreeNode) bpelVariablesOutputTreeModel.getRoot();
        faultTreeTopTreeNode = (DefaultMutableTreeNode) bpelVariablesOutputTreeModel.getRoot();
        initComponents();
    }

    @Override
    public String getName() {
        return "Input/Output/Fault Message";
    }

    public Message getInputMessageType() {
        return inputMessageType;
    }

    public Message getOutputMessageType() {
        return outputMessageType;
    }
    
    public Message getFaultMessageType() {
        return faultMessageType;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        jTextFieldInputVariable = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jTextFieldOutputVariable = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTreeInputVariable = new javax.swing.JTree(inputTreeTopTreeNode);
        jTreeInputVariable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        jScrollPane3 = new javax.swing.JScrollPane();
        jTreeOutputVariable = new javax.swing.JTree(outputTreeTopTreeNode);
        jTreeOutputVariable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        jScrollPane4 = new javax.swing.JScrollPane();
        jTreeFaultVariable = new javax.swing.JTree(faultTreeTopTreeNode);
        jTreeFaultVariable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jTextFieldFaultVariable = new javax.swing.JTextField();

        org.openide.awt.Mnemonics.setLocalizedText(jLabel9, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel9.text_1")); // NOI18N

        jTextFieldInputVariable.setEditable(false);
        jTextFieldInputVariable.setText(org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jTextFieldInputVariable.text_1")); // NOI18N
        jTextFieldInputVariable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldInputVariableActionPerformed(evt);
            }
        });

        org.openide.awt.Mnemonics.setLocalizedText(jLabel5, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel5.text")); // NOI18N

        org.openide.awt.Mnemonics.setLocalizedText(jLabel10, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel10.text_1")); // NOI18N

        jTextFieldOutputVariable.setEditable(false);
        jTextFieldOutputVariable.setText(org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jTextFieldOutputVariable.text_1")); // NOI18N
        jTextFieldOutputVariable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldOutputVariableActionPerformed(evt);
            }
        });

        org.openide.awt.Mnemonics.setLocalizedText(jLabel6, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel6.text")); // NOI18N

        jTreeInputVariable.setMaximumSize(new java.awt.Dimension(75, 65));
        jTreeInputVariable.setModel(bpelVariablesInputTreeModel);
        jTreeInputVariable.setPreferredSize(new java.awt.Dimension(75, 65));
        jTreeInputVariable.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                jTreeInputVariableValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(jTreeInputVariable);

        jTreeOutputVariable.setMaximumSize(new java.awt.Dimension(75, 65));
        jTreeOutputVariable.setPreferredSize(new java.awt.Dimension(75, 65));
        jTreeOutputVariable.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                jTreeOutputVariableValueChanged(evt);
            }
        });
        jScrollPane3.setViewportView(jTreeOutputVariable);

        jTreeFaultVariable.setMaximumSize(new java.awt.Dimension(75, 65));
        jTreeFaultVariable.setPreferredSize(new java.awt.Dimension(75, 65));
        jTreeFaultVariable.addTreeSelectionListener(new javax.swing.event.TreeSelectionListener() {
            public void valueChanged(javax.swing.event.TreeSelectionEvent evt) {
                jTreeFaultVariableValueChanged(evt);
            }
        });
        jScrollPane4.setViewportView(jTreeFaultVariable);

        org.openide.awt.Mnemonics.setLocalizedText(jLabel1, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel1.text")); // NOI18N

        org.openide.awt.Mnemonics.setLocalizedText(jLabel2, org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jLabel2.text")); // NOI18N

        jTextFieldFaultVariable.setText(org.openide.util.NbBundle.getMessage(WLMVisualPanel2.class, "WLMVisualPanel2.jTextFieldFaultVariable.text")); // NOI18N

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(248, 248, 248)
                        .add(jLabel8))
                    .add(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                                .add(jLabel9)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldInputVariable, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 126, Short.MAX_VALUE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jLabel10)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldOutputVariable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 122, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 177, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jScrollPane3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 173, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                                .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 200, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jLabel6)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(52, 52, 52)
                                .add(jLabel1))
                            .add(jPanel1Layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 178, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(jLabel2)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldFaultVariable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 126, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(jLabel6)
                    .add(jLabel1))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(jScrollPane1, 0, 0, Short.MAX_VALUE)
                    .add(jScrollPane3, 0, 0, Short.MAX_VALUE)
                    .add(jScrollPane4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 201, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel9)
                    .add(jTextFieldInputVariable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel10)
                    .add(jTextFieldOutputVariable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel2)
                    .add(jTextFieldFaultVariable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(98, 98, 98)
                .add(jLabel8))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(24, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

private void jTextFieldInputVariableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldInputVariableActionPerformed
// TODO add your handling code here:
}//GEN-LAST:event_jTextFieldInputVariableActionPerformed

private void jTextFieldOutputVariableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldOutputVariableActionPerformed
// TODO add your handling code here:
}//GEN-LAST:event_jTextFieldOutputVariableActionPerformed

private void jTreeInputVariableValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_jTreeInputVariableValueChanged
    Object object = getUserObject(evt);
    if (object == null) {
        return;
    }
    inputMessageType = ((VariableType) object).getMessage();
    updateSelection(object, jTextFieldInputVariable);
}//GEN-LAST:event_jTreeInputVariableValueChanged

private void jTreeOutputVariableValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_jTreeOutputVariableValueChanged
    Object object = getUserObject(evt);
    if (object == null) {
        return;
    }
    outputMessageType = ((VariableType) object).getMessage();   
    updateSelection(object, jTextFieldOutputVariable);
}//GEN-LAST:event_jTreeOutputVariableValueChanged

private void jTreeFaultVariableValueChanged(javax.swing.event.TreeSelectionEvent evt) {//GEN-FIRST:event_jTreeFaultVariableValueChanged
    Object object = getUserObject(evt);
    if (object == null) {
        return;
    }
    faultMessageType = ((VariableType) object).getMessage();   
    updateSelection(object, jTextFieldFaultVariable);
}//GEN-LAST:event_jTreeFaultVariableValueChanged

private void updateSelection(Object object, JTextField tf) {
   if (object instanceof VariableType) {
       String name = ((VariableType) object).getVariableName();
       tf.setText("$" + name);
   }
}
    
private Object getUserObject(javax.swing.event.TreeSelectionEvent evt) {
    DefaultMutableTreeNode node = (DefaultMutableTreeNode) evt.getNewLeadSelectionPath().getLastPathComponent();
    if (node == null) {
        return null;
    }
    return node.getUserObject();
}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JTextField jTextFieldFaultVariable;
    private javax.swing.JTextField jTextFieldInputVariable;
    private javax.swing.JTextField jTextFieldOutputVariable;
    private javax.swing.JTree jTreeFaultVariable;
    private javax.swing.JTree jTreeInputVariable;
    private javax.swing.JTree jTreeOutputVariable;
    // End of variables declaration//GEN-END:variables
}

